/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/




#import <math.h>
#import "Cell.h"
#import "HabitatSpace.h"
#import "Trout.h"
#import "globals.h"


@implementation Cell

+ createBegin: aZone 
{
  Cell * newCell;

  newCell = [super createBegin: aZone];

  newCell->cellZone = [Zone create: aZone];

  newCell->space = nil;
  newCell->myRandGen = nil;
  newCell->refPoint=nil;
  newCell->midPoint=nil;
  newCell->extent=nil;
  newCell->boundary=(Vector **) nil;
  newCell->depth=LARGEINT;
  newCell->area=LARGEINT;
  newCell->cellFracSpawn=LARGEINT;
  newCell->cellFracShelter=LARGEINT;
  newCell->cellShelterArea=LARGEINT;
  newCell->fishIContain=nil;
  newCell->myColor=0;
  newCell->hydStation=nil;

  newCell->fishParamsMap=nil;

  newCell->tagCellColor = '\0';

  newCell->isShelterAvailable = NO;

  return newCell;
}




///////////////////////////////////////////////
//
// createEnd
//
//////////////////////////////////////////////
- createEnd 
{
  [super createEnd];

  if(myRandGen == nil)
  {
     fprintf(stderr, "ERROR: Cell >>>> createEnd >>>> myRandGen is nil\n");
     fflush(0);
     exit(1);
  } 

  //
  // misc initializations
  //
  fishIContain  = [List create: cellZone];
  reddsIContain = [List create: cellZone];
  listOfAdjacentCells = [List create: cellZone];

  if(fishParamsMap == nil)
  {
     fprintf(stderr, "ERROR: Cell >>>> createEnd >>>> fishParamsMap is nil\n");
     fflush(0);
     exit(1);
  }

  [self initializeSurvProb];

  //foodReportFirstTime=YES;
  //depthVelRptFirstTime=YES;
 
  return self;
}


////////////////////////////////////
//
// setRandGen
//
//////////////////////////////////
- setGenerator: aRandGen
{
    myRandGen = aRandGen;
    return self;
}

//////////////////////////////////////
//
// getRandGen
//
//////////////////////////////////////
- getRandGen
{
    return myRandGen;
}

////////////////////////////////////////////////
//
// setRaster* 
//
///////////////////////////////////////////
- setRasterResolution: (int) aResolution {
  rasterResolution = aResolution;
  return self;
}

- setRasterResolutionX: (int) aResolutionX {
  rasterResolutionX = aResolutionX;
  return self;
}

- (int) getRasterResolutionX {
  return rasterResolutionX;
}

- (int) getRasterResolutionY {
  return rasterResolutionY;
}

- setRasterResolutionY: (int) aResolutionY {
  rasterResolutionY = aResolutionY;
  return self;
}

- (int) getRasterResolution {
  return rasterResolution;
}

- setRasterColorVariable: (char *) aColorVariable {

  rasterColorVariable = aColorVariable;

  return self;
}

/////////////////////////////////////////////////////////////////////
//
// setSpace
//
////////////////////////////////////////////////////////////////////
- setSpace: aSpace {
  space = aSpace;
  return self;
}




////////////////////////////////////////////////////////////////////
//
// getSpace
//
///////////////////////////////////////////////////////////////////
- getSpace {
  return space;
}



/////////////////////////////////
//
// setReach
//
////////////////////////////////
- setReach: aReach
{
   reach = aReach;
   return self;
}


//////////////////////////////////
//
// getReach
//
/////////////////////////////////
- getReach
{
   return reach;
}


/////////////////////////////////////////////////////
//
// setTimeManager
//
/////////////////////////////////////////////////////
- setTimeManager: (id <TimeManager>) aTimeManager
{
     timeManager = aTimeManager;
  
     return self;
}

/////////////////////////////////////////////
//
// setModel
//
/////////////////////////////////////////////
- setModel: (id <TroutModelSwarm>) aModel
{
    model = (id <TroutModelSwarm>) aModel;
    return self;
}


//
// Added for the survival manager
//
//////////////////////////////////////
//
// setFishParamsMap
//
/////////////////////////////////////
- setFishParamsMap: (id <Map>) aMap
{
    fishParamsMap = aMap;
    return self;
}


- setNumberOfSpecies: (int) aNumberOfSpecies
{
    numberOfSpecies = aNumberOfSpecies;
    return self;
}



//////////////////////////////////////////////
//
// setHabShearParamA:habShearParamB
//
////////////////////////////////////////////
- setHabShearParamA: (double) aHabShearParamA
     habShearParamB: (double) aHabShearParamB
{
    habShearParamA = aHabShearParamA;
    habShearParamB = aHabShearParamB;
    return self;
}


////////////////////////////////////////
//
// getHabShearParamA
//
////////////////////////////////////////
- (double) getHabShearParamA
{
   return habShearParamA;
}


////////////////////////////////////////
//
// getHabShearParamB
//
////////////////////////////////////////
- (double) getHabShearParamB
{
   return habShearParamB;
}

////////////////////////////////////////////////////
//
// setHabSheltSpeedFrac
//
////////////////////////////////////////////////////
- setHabShelterSpeedFrac: (double) aShelterSpeedFrac
{
    habShelterSpeedFrac = aShelterSpeedFrac;
    return self;
}


///////////////////////////////////////////////
//
// getHabShelterSpeedFrac
//
///////////////////////////////////////////////
- (double) getHabShelterSpeedFrac
{
   return habShelterSpeedFrac;
}



///////////////////////////////////////////////////////////////
//
// setTransect
//
///////////////////////////////////////////////////////////////
- setTransect: (int) aTransect {
  transectNumber = aTransect;
  return self;
}



/////////////////////////////////////////////////////////////////
//
// getTransect
//
////////////////////////////////////////////////////////////////
- (int)  getTransect {
 return transectNumber;
}



/////////////////////////////////////////////////////////////////
//
// setRefPoint
// Upper left (facing downstream) corner of cell
//
////////////////////////////////////////////////////////////////
- setRefPoint: (Vector *) aPoint {
  refPoint = aPoint;
  return self;
}


///////////////////////////////////////////////////////////////////
//
// getRefPoint
//
//////////////////////////////////////////////////////////////////
- (Vector *) getRefPoint {
  return refPoint;
}

/////////////////////////////////////////////////////////////////
//
// setMidPoint
// Note: Calculated in HabitatSpace
//
//////////////////////////////////////////////////////////////////
- setMidPoint: (Vector *) aPoint {
   midPoint = aPoint;
   return self;
}


////////////////////////////////////////////////////////////
//
// setHydSatation
//
///////////////////////////////////////////////////////////
- setHydStation: aStation
{
  hydStation = aStation;
  return self;
}

//////////////////////////////////////////////
//
// setBottomElev
//
/////////////////////////////////////////////
- setBottomElev: (double) anElev
{
   bottomElev = anElev;
   return self;
}


///////////////////////////////////////////////////////
//
// checkThatHydStationIsNotNil 
//
//////////////////////////////////////////////////////
- checkThatHydStationIsNotNil 
{
   if(hydStation == nil) 
   {
     fprintf(stderr, "ERROR: hydStation is nil in transect %d cellNo %d\n", transectNumber, cellNumber);
     fflush(0);
     exit(1);
   }

   return self;
}



///////////////////////////////////////////////
//
// getMidPoint
//
//////////////////////////////////////////////
- (Vector *) getMidPoint {
  return midPoint;
}


////////////////////////////////////////////////////////
//
// setDistanceToHide
//
///////////////////////////////////////////////////////
- setDistanceToHide: (double) aDistance {
  cellDistToHide = aDistance;
  return self;
}


//////////////////////////////////////////////////////
//
// getDistanceToHide
//
/////////////////////////////////////////////////////
- (double) getDistanceToHide {

  return cellDistToHide;

}




//////////////////////////////////////////////////////
//
// setCellNo
//
/////////////////////////////////////////////////////////
- setCellNo: (int) aCellNo 
{
  cellNumber = aCellNo;
  return self;
}

//////////////////////////////////////////////////////
//
// getCellNo
//
/////////////////////////////////////////////////////
- (int) getCellNo {
  return cellNumber;
}




/////////////////////////////////////////////////////////////////////
//
// setExtent
// Extent is the length and width of a cell
// Used to calculate the area of a cell
//
/////////////////////////////////////////////////////////////////////
- setExtent: (Vector *) aPoint {
  extent = aPoint;
  return self;
}




///////////////////////////////////////////////////////////////////
//
// getExtent
//
///////////////////////////////////////////////////////////////////
- (Vector *) getExtent {
  return extent;
}




//////////////////////////////////////////////////////////////////
//
// setBoundary
//
//////////////////////////////////////////////////////////////////
- setBoundary: (Vector **) aVector {
  // accept the boundary values from model

  // this section relies on dynamically allocated pointer in HabitatSpace.m
  boundary = aVector;
  


  // calculate area for this Cell
  area = [self getXExtent]*[self getYExtent];
  return self;
}




///////////////////////////////////////////////////////////////////////
//
// getBoundary
//
//////////////////////////////////////////////////////////////////////
- (Vector **) getBoundary {
  return boundary;
}




/////////////////////////////////////////////////////////////////////
//
// getArea
//
/////////////////////////////////////////////////////////////////////
- (double) getArea {
  return area;
}




/////////////////////////////////////////////////////////////////////
//
// getXExtent
//
////////////////////////////////////////////////////////////////////
- (double) getXExtent {
 return ([boundary[1] getElement: XCOORDINATE] - 
          [boundary[0] getElement: XCOORDINATE]);
}



/////////////////////////////////////////////////////////////////////
//
// getYExtent
//
////////////////////////////////////////////////////////////////////
- (double) getYExtent {
  return ([boundary[2] getElement: YCOORDINATE] -
          [boundary[0] getElement: YCOORDINATE]);
}



/////////////////////////////////////////////////////////////
//
// containsX
//
////////////////////////////////////////////////////////////
- (BOOL) containsX: (int) anInt {

  return (([boundary[0] getElement: XCOORDINATE] <= anInt) &&
           (anInt <= [boundary[3] getElement: XCOORDINATE]) );
}





/////////////////////////////////////////////////////////////
//
// containsY
//
////////////////////////////////////////////////////////////
- (BOOL) containsY: (int) anInt 
{
  return (([boundary[0] getElement: YCOORDINATE] <= anInt) &&
           (anInt <= [boundary[3] getElement: YCOORDINATE]));
}


/////////////////////////////////////////////////////////////
//
// containsDoubleX
//
////////////////////////////////////////////////////////////
- (BOOL) containsDoubleX: (double) aDouble 
{
  return (([boundary[0] getElement: XCOORDINATE] <= aDouble) &&
           (aDouble <= [boundary[3] getElement: XCOORDINATE]) );
}





/////////////////////////////////////////////////////////////
//
// containsDoubleY
//
////////////////////////////////////////////////////////////
- (BOOL) containsDoubleY: (double) aDouble 
{
  return (([boundary[0] getElement: YCOORDINATE] <= aDouble) &&
           (aDouble <= [boundary[3] getElement: YCOORDINATE]));
}

#define COLOR_MODIFIER 1.8679
/////////////////////////////////////////////////////////////////////
//
// drawSelfOn
//
/////////////////////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
  double colorVariable=0.0;
  int i, numfishincell;
  int j, numReddsInCell;
  id <ListIndex> numFishNdx;
  id <ListIndex> numReddsNdx;
  id fish;
  id redd;

  // first calculate color to use
  // [0,100] use continous scale
  // [101,1000] use bin values

    if(strcmp("depth",rasterColorVariable) == 0) 
    {
      colorVariable = depth; 
    }
    else if(strcmp("velocity",rasterColorVariable) == 0) 
    {
    colorVariable = velocity; 
    }
    else 
    {
        fprintf(stderr, "ERROR: Unknown rasterColorVariable value = %s\n",rasterColorVariable);
        fflush(0);
        exit(1);
    }

  if ((0.0 <= colorVariable) && (colorVariable <= 100.0))
    myColor = (int)(colorVariable/COLOR_MODIFIER + 0.5);
  else if ((100.0 < colorVariable) && (colorVariable <= 150.0))
    myColor = 55L;
  else if ((150.0 < colorVariable) && (colorVariable <= 200.0))
    myColor = 56L;
  else if ((200.0 < colorVariable) && (colorVariable <= 250.0))
    myColor = 57L;
  else if ((250.0 < colorVariable) && (colorVariable <= 300.0))
    myColor = 58L;
  else if ((300.0 < colorVariable) && (colorVariable <= 350.0))
    myColor = 59L;
  else if ((350.0 < colorVariable) && (colorVariable <= 400.0))
    myColor = 60L;
  else if ((400.0 < colorVariable) && (colorVariable <= 450.0))
    myColor = 61L;
  else if ((450.0 < colorVariable) && (colorVariable <= 500.0))
    myColor = 62L;
  else if (500.0 < colorVariable)
    myColor = 63L;
  else
    myColor = 0L;

  // put a lower bound on the color
  if (myColor <= 0)
  {
      myColor = 1L;
  }

  if(tagCell)
  {
      myColor = TAG_CELL_COLOR;
  }

  [aRaster fillRectangleX0: 
               (int)([boundary[0] getElement: XCOORDINATE]/rasterResolutionX + 0.5)
           Y0: (int)([boundary[0] getElement: YCOORDINATE]/rasterResolutionY + 0.5)
           X1: (int)([boundary[3] getElement: XCOORDINATE]/rasterResolutionX + 0.5)
           Y1: (int)([boundary[3] getElement: YCOORDINATE]/rasterResolutionY + 0.5)
           Color: myColor ];

  [aRaster rectangleX0: (int)([boundary[0] getElement: XCOORDINATE]/rasterResolutionX + 0.5)
           Y0: (int)([boundary[0] getElement: YCOORDINATE]/rasterResolutionY + 0.5)
           X1: (int)([boundary[3] getElement: XCOORDINATE]/rasterResolutionX + 0.5)
           Y1: (int)([boundary[3] getElement: YCOORDINATE]/rasterResolutionY + 0.5)
           Width: 1
           Color: 0L ];






  // draw my fish

  numfishincell = [fishIContain getCount];
  numFishNdx = [fishIContain listBegin: scratchZone];
      if(numfishincell != 0) {
             i=1;
             while( ([numFishNdx getLoc] != End) && ((fish = [numFishNdx next]) != nil) ) {
              [fish drawSelfOn: aRaster atPosition: i  of: numfishincell];
              i++;
             }
      }
  [numFishNdx drop];





  // draw my Redds


  numReddsInCell = [reddsIContain getCount];
  numReddsNdx = [reddsIContain listBegin: scratchZone];
      if(numReddsInCell != 0) 
      {
             j=1;

             while( ([numReddsNdx getLoc] != End) && ((redd = [numReddsNdx next]) != nil) ) 
             {
                 [redd drawSelfOn: aRaster atPosition: j  of: numReddsInCell];
                 j++;
             }
      }
  [numReddsNdx drop];

  tagCell = NO;

  return self;
}


/////////////////////////////////
//
// tagDestCells
//
////////////////////////////////
- tagDestCells
{
    tagCell = YES;
    return self;
}


///////////////////////////////////////////////
//
// getNeighborsWithin
//
//////////////////////////////////////////////
- getNeighborsWithin: (double) aRange 
            withList: (id <List>) aCellList
{
  [space getNeighborsWithin: aRange 
                         of: self
                   withList: aCellList];
  return self;
}


///////////////////////////////
//
// getListOfAdjacentCells
//
//////////////////////////////
- (id <List>) getListOfAdjacentCells 
{
   return listOfAdjacentCells; 
}



/////////////////////
//
// getNumberOfFish
//
/////////////////////
- (int) getNumberOfFish {
  return [fishIContain getCount];
}




//////////////////////////////////////////////////////////////////
//
// getNumberOfRedds
//
/////////////////////////////////////////////////////////////////
- (int) getNumberOfRedds {
  return [reddsIContain getCount];
}




//////////////////////////////////////////////////////////////////
//
// getFishIContain
//
/////////////////////////////////////////////////////////////////
- (id <List>) getFishIContain {
  return fishIContain;
}




//////////////////////////////////////////////////////////////////
//
// getReddsYouContain
//
/////////////////////////////////////////////////////////////////
- (id <List>) getReddsYouContain {
  return reddsIContain;
}



///////////////////////////////////////////////////////////////////
//
// setVelocityInterpolator
//
///////////////////////////////////////////////////////////////////
- setVelocityInterpolator: (id <InterpolationTable>) anInterpolator
{
   velocityInterpolator = anInterpolator;
   return self;
}



////////////////////////////////////////////////////////////////
//
// setWslInterpolator
//
////////////////////////////////////////////////////////////////
- setWslInterpolator: (id <InterpolationTable>) anInterpolator
{
    wslInterpolator = anInterpolator;
    return self;
}


///////////////////////////////////////////////////////////////////
//
// calcDepthAndVelocityWithIndex
//
///////////////////////////////////////////////////////////////////
- calcDepthAndVelocityWithIndex: (int) anInterpolationIndex
             withInterpFraction: (double) anInterpFraction
{
   double aVelocity = -999999.0;
   double aDepth = -999999.0;
   double aWsl = -999999.0;
 
   aVelocity = [velocityInterpolator getValueWithTableIndex: anInterpolationIndex 
                                         withInterpFraction: anInterpFraction];

   aWsl = [wslInterpolator getValueWithTableIndex: anInterpolationIndex
                                   withInterpFraction: anInterpFraction];

   //fprintf(stdout, "Cell >>>> calcDepthAndVelocityWithIndex >>>> velocity = %f\n", velocity);
   //fflush(0);

   // The interpolators can produce negative depths and velocities
   // for flows less than the lowest one in the hydraulic data files.
   // These are set to zero. 



   // fprintf(stdout, "Cell >>>> calcDepthAndVelocityWithIndex >>>> aWsl = %f\n", aWsl);
   // fprintf(stdout, "Cell >>>> calcDepthAndVelocityWithIndex >>>> bottomElev = %f\n", bottomElev);
   // fprintf(stdout, "Cell >>>> calcDepthAndVelocityWithIndex >>>> aDepth = %f\n", aDepth);
   // fflush(0);

   aDepth = aWsl - bottomElev;

   if(aDepth < 0.0)
    {
      aDepth = 0.0;
    }

   if(aVelocity < 0.0) 
   {
     aVelocity = 0.0;
   }

   velocity = aVelocity;
   depth = aDepth;

   return self;
}


///////////////////////////////////////////
//
// getYesterdaysRiverFlow
//
/////////////////////////////////////////
- (double) getYesterdaysRiverFlow {

  return [space getYesterdaysRiverFlow];

}


///////////////////////////////////////////
//
// getRiverFlow
//
/////////////////////////////////////////
- (double) getRiverFlow {

  return [space getRiverFlow];

}


////////////////////////////////////////////////////////////////////
//
// getTomrrowsRiverFlow
//
////////////////////////////////////////////////////////////////////
- (double) getTomorrowsRiverFlow {

  return [space getTomorrowsRiverFlow];

}

//////////////////////////////////////////////////////////////
//
// setCellVelocity
//
/////////////////////////////////////////////////////////////
- setCellVelocity : (double) newVelocity {
velocity = newVelocity;
return self;
}



//////////////////////////////////////////////////////////////////
//
// getCellVelocity
//
/////////////////////////////////////////////////////////////////
- (double) getCellVelocity {
   return velocity;
}



////////////////////////////////////////////////////////////////////
//
// getFlowChange
//
////////////////////////////////////////////////////////////////////
- (double) getFlowChange {
  return [space getFlowChange];
}




//////////////////////////////////////////////////////////////////
//
// setCellFracShelter
//
//////////////////////////////////////////////////////////////////
- (void) setCellFracShelter: (double) aFloat {
  cellFracShelter = aFloat;
}


/////////////////////////////////////////////////////////////////
//
// calcCellShelterArea
//
////////////////////////////////////////////////////////////////
- (void) calcCellShelterArea 
{
  cellShelterArea = [self getArea]*cellFracShelter;
}

/////////////////////////////////////
//
// resetShelterAreaAvailable
//
////////////////////////////////////////
- (void) resetShelterAreaAvailable 
{
   shelterAreaAvailable = cellShelterArea;
 
   if(shelterAreaAvailable > 0.0)
   {
       isShelterAvailable = YES;
   }
   else
   {
       isShelterAvailable = NO;
   }
}


////////////////////////////////////
//
// getShelterAreaAvailable 
//
////////////////////////////////////
- (double) getShelterAreaAvailable 
{
  return shelterAreaAvailable;
}

//////////////////////////////////
//
// getIsShelterAvailable
//
//////////////////////////////////
- (BOOL) getIsShelterAvailable
{
   return isShelterAvailable;
}


//////////////////////////////////////////////////////////////////
//
// setCellFracSpawn
//
//////////////////////////////////////////////////////////////////
- setCellFracSpawn: (double) aFloat 
{
  cellFracSpawn = aFloat;
  return self;
}





////////////////////////////////////////////////////////////////////
//
// getCellFracSpawn
//
////////////////////////////////////////////////////////////////////
- (double) getCellFracSpawn {
   return cellFracSpawn;
}


/////////////////////////////////////////////////////
//
// getCellFracShelter
//
/////////////////////////////////////////////////////
- (double) getCellFracShelter
{
    return cellFracShelter;
}

//////////////////////////////////////////////////////
//
// spawnHere
//
/////////////////////////////////////////////////////
- spawnHere: aFish {
  [self addFish: aFish];
  return self;
}

/////////////////////////////////////////////////////////////
//
// eatHere
//
/////////////////////////////////////////////////////////////
- eatHere: aFish 
{
  //
  // sheltered?
  //
  if(shelterAreaAvailable > 0.0) 
  {
    if([aFish getAmIInAShelter] == YES ) 
    {
        shelterAreaAvailable -= [aFish getFishShelterArea];
    }
    if(shelterAreaAvailable < 0.0) 
    {
         shelterAreaAvailable = 0.0;
         isShelterAvailable = NO;
    }
  }

  hourlyAvailDriftFood -= [aFish getHourlyDriftConRate];
  hourlyAvailSearchFood -= [aFish getHourlySearchConRate];

  [self addFish: aFish];

#ifdef FOODAVAILREPORT
  [self foodAvailAndConInCell: aFish];
#endif

  return self;
}


/////////////////////////////////////////////////////////////////////
//
// addFish
//
//
/////////////////////////////////////////////////////////////////////
- addFish: aFish 
{
  id fishOldCell=nil;

  fishOldCell = [aFish getCell];

  if(fishOldCell != nil) [fishOldCell removeFish: aFish];
   
  [fishIContain addLast: aFish];
  [aFish setCell: self];

  [aFish setReach: reach];

  return self;
}




/////////////////////////////////////////////////////////////////////////
//
// removeFish
//
/////////////////////////////////////////////////////////////////////////
- removeFish: aFish 
{

  [fishIContain remove: aFish];
  [aFish setCell: nil];

  return self;
}




///////////////////////////////////////////////////////////////
//
// addRedd
//
//
// addRedd has a different functionality from addFish.  Since
// redds don't move around like fish, they spend their entire
// life in once cell.  So, an "addredd" only occurs after the
// creation of a new redd. 
//
// NOTE: The new Redd MUST BE add to the BEGINNING of the
//       reddIContain List in order to make the SUPER IMPOSITION
//       function work.
//
//
///////////////////////////////////////////////////////////////
- addRedd: aRedd 
{
  [reddsIContain addFirst: aRedd];
  return self;
}




/////////////////////////////////////////////////////////////
//
// removeRedd
//
/////////////////////////////////////////////////////////////
- removeRedd: aRedd 
{
  [reddsIContain remove: aRedd];
  return self;
}



////////////////////////////////////////////////
//
// getHabSearchProd
//
////////////////////////////////////////////////
- (double) getHabSearchProd {
  return [space getHabSearchProd];
}



//////////////////////////////////////////////
//
// getHabDriftConc
//
/////////////////////////////////////////////
- (double) getHabDriftConc {
  return [space getHabDriftConc];
}

/////////////////////////////////////////
//
// getHabDriftRegenDist
//
/////////////////////////////////////////
- (double) getHabDriftRegenDist {
   return [space getHabDriftRegenDist];
}

//////////////////////////////////////////////////////////
//
// getHabPreyEnergyDensity
//
/////////////////////////////////////////////////////////
- (double) getHabPreyEnergyDensity {
  return [space getHabPreyEnergyDensity];
}



//////////////////////////////////////////////////
//
// getTemperature
//
//////////////////////////////////////////////////
- (double) getTemperature {
  return [space getTemperature];
}


//////////////////////////////////////////////////
//
// getTurbidity
//
//////////////////////////////////////////////////
- (double) getTurbidity 
{
  return [space getTurbidity];
}


/////////////////////////////////////////////
//
// calcDriftHourlyTotal
//
////////////////////////////////////////////
-  calcDriftHourlyTotal {
  driftHourlyCellTotal = 3600*[self getYExtent]*depth*velocity*[self getHabDriftConc]
                         *([self getXExtent]/[self getHabDriftRegenDist]);
 return self;

}


////////////////////////////////////////////
//
// calcSearchHourlyTotal
//
//////////////////////////////////////////////
- calcSearchHourlyTotal {

  searchHourlyCellTotal = area*[self getHabSearchProd];
  return self;
}


//////////////////////////////////////////
//
// getHourlyAvailDriftFood
//
////////////////////////////////////////
- (double) getHourlyAvailDriftFood {
   return hourlyAvailDriftFood;
}

//////////////////////////////////////////
//
// getHourlyAvailSearchFood
//
////////////////////////////////////////
- (double) getHourlyAvailSearchFood {
   return hourlyAvailSearchFood;
}

//////////////////////////////////
//
// updateDSCellHourlyTotal
//
//////////////////////////////////
- (void) updateDSCellHourlyTotal {
  [self calcDriftHourlyTotal];
  [self calcSearchHourlyTotal];
}


/////////////////////////////////////
//
//resetAvailHourlyTotal
//
//////////////////////////////////////
- (void) resetAvailHourlyTotal {
  hourlyAvailDriftFood = driftHourlyCellTotal;
  hourlyAvailSearchFood = searchHourlyCellTotal;
}
////////////////////////////////////////////////
//
// getDayLength
//
////////////////////////////////////////////////
- (double) getDayLength {
  return [space getDayLength];
}




////////////////////////////////////////////////
//
// getDepth
//
////////////////////////////////////////////////
- (double) getDepth {
  return depth;
}



//////////////////////////////////////////
//
// isDepthGreaterThan0
//
/////////////////////////////////////////
- (BOOL) isDepthGreaterThan0
{
    if(depth <= 0.0)
    {
      return NO;
    }

    return YES;
}


////////////////////////////////////////////////
//
// setDepth
//
////////////////////////////////////////////////
- setDepth: (double) aFloat {
  depth = aFloat;
  return self;
}


/////////////////////////////////////
//
// getPiscivorousFishDensity
//
////////////////////////////////////
- (double) getPiscivorousFishDensity
{
   return [space calcPiscivorousFishDensity];
} 


////////////////////////////////////////////////
//
// initializeSurvProb
//
////////////////////////////////////////////////
- initializeSurvProb 
{
  id <Index> mapNdx;
  FishParams* fishParams = nil;

  //fprintf(stdout, "Cell >>>> initializeSurvProb >>>> BEGIN\n");
  //fflush(0);

  if(numberOfSpecies <= 0)
  {
     fprintf(stderr, "ERROR: Cell >>>> initializeSurvProb >>>> numberOfSpecies is 0\n");
     fflush(0);
     exit(1);
  }

  survMgrMap = [Map create: cellZone];
  survMgrReddMap = [Map create: cellZone];

  mapNdx = [fishParamsMap mapBegin: scratchZone];

  while(([mapNdx getLoc] != End) && ((fishParams = (FishParams *) [mapNdx next]) != nil))
  {

     id <SurvMGR> survMgr;
     id <Symbol> species = [fishParams getFishSpecies];
 
     survMgr = [SurvMGR     createBegin: cellZone
                     withHabitatObject: self];

     [survMgrMap at: species  insert: survMgr];

     ANIMAL = [survMgr getANIMALSYMBOL];
     HABITAT = [survMgr getHABITATSYMBOL];

      //
      // High Temperature
      //

      [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "HighTemperature"] 
                        withType: "SingleFunctionProb"
                  withAgentKnows: YES
                 withIsStarvProb: NO];

      [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "HighTemperature"] 
                           withInputObjectType: HABITAT
                             withInputSelector: M(getTemperature)
                                   withXValue1: fishParams->mortFishHiTT9
                                   withYValue1: 0.9
                                   withXValue2: fishParams->mortFishHiTT1
                                   withYValue2: 0.1];


      [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "Velocity"] 
                        withType: "SingleFunctionProb"
                  withAgentKnows: YES
                 withIsStarvProb: NO];

      [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "Velocity"] 
                           withInputObjectType: ANIMAL
                             withInputSelector: M(getSwimSpeedMaxSwimSpeedRatio)
                                   withXValue1: fishParams->mortFishVelocityV9
                                   withYValue1: 0.9
                                   withXValue2: fishParams->mortFishVelocityV1
                                   withYValue2: 0.1];

      [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "Stranding"] 
                        withType: "SingleFunctionProb"
                  withAgentKnows: YES
                 withIsStarvProb: NO];

      [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "Stranding"] 
                           withInputObjectType: ANIMAL
                             withInputSelector: M(getDepthLengthRatioForCell)
                                   withXValue1: fishParams->mortFishStrandD1
                                   withYValue1: 0.1
                                   withXValue2: fishParams->mortFishStrandD9
                                   withYValue2: 0.9];

     //
     // Poor Condition
     //

     [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "PoorCondition"]
                       withType: "SingleFunctionProb"
                 withAgentKnows: YES
                withIsStarvProb: YES];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "PoorCondition"] 
                          withInputObjectType: ANIMAL
                            withInputSelector: M(getFishCondition)
                                  withXValue1: fishParams->mortFishConditionK1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishConditionK9
                                  withYValue2: 0.9];
    

     //
     // TerrestialPredation Predation
     // 
     [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
             withType: "LimitingFunctionProb"
       withAgentKnows: YES
      withIsStarvProb: NO];

     [survMgr addConstantFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withValue: fishParams->mortFishTerrPredMin];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getDepth)
                                  withXValue1: fishParams->mortFishTerrPredD1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishTerrPredD9
                                  withYValue2: 0.9];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getTurbidity)
                                  withXValue1: fishParams->mortFishTerrPredT1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishTerrPredT9
                                  withYValue2: 0.9];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: ANIMAL
                            withInputSelector: M(getFishLength)
                                  withXValue1: fishParams->mortFishTerrPredL9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishTerrPredL1
                                  withYValue2: 0.1];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: ANIMAL
                            withInputSelector: M(getFeedTimeForCell)
                                  withXValue1: fishParams->mortFishTerrPredF9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishTerrPredF1
                                  withYValue2: 0.1];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getCellVelocity)
                                  withXValue1: fishParams->mortFishTerrPredV1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishTerrPredV9
                                  withYValue2: 0.9];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "TerrestialPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getDistanceToHide)
                                  withXValue1: fishParams->mortFishTerrPredH9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishTerrPredH1
                                  withYValue2: 0.1];





     //
     // Aquatic Predation
     // 
     [survMgr addPROBWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
             withType: "LimitingFunctionProb"
       withAgentKnows: YES
      withIsStarvProb: NO];


     [survMgr addConstantFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withValue: fishParams->mortFishAqPredMin];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getDepth)
                                  withXValue1: fishParams->mortFishAqPredD9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishAqPredD1
                                  withYValue2: 0.1];



     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: ANIMAL
                            withInputSelector: M(getFishLength)
                                  withXValue1: fishParams->mortFishAqPredL1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishAqPredL9
                                  withYValue2: 0.9];


     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: ANIMAL
                            withInputSelector: M(getFeedTimeForCell)
                                  withXValue1: fishParams->mortFishAqPredF9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishAqPredF1
                                  withYValue2: 0.1];

     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getPiscivorousFishDensity)
                                  withXValue1: fishParams->mortFishAqPredP9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishAqPredP1
                                  withYValue2: 0.1];


     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getTurbidity)
                                  withXValue1: fishParams->mortFishAqPredU1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortFishAqPredU9
                                  withYValue2: 0.9];


     [survMgr addLogisticFuncToProbWithSymbol: [model getFishMortalitySymbolWithName: "AquaticPredation"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getTemperature)
                                  withXValue1: fishParams->mortFishAqPredT9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortFishAqPredT1
                                  withYValue2: 0.1];

             

     [survMgr setLogisticFuncLimiterTo: 20.0];
     //[survMgr setTestOutputOnWithFileName: "SurvMGRTest.out"];
     survMgr = [survMgr createEnd];

  }
 
  [mapNdx setLoc: Start];

  while(([mapNdx getLoc] != End) && ((fishParams = (FishParams *) [mapNdx next]) != nil))
  {

     id <SurvMGR> survMgr;
     id <Symbol> species = [fishParams getFishSpecies];
 
     survMgr = [SurvMGR     createBegin: cellZone
                     withHabitatObject: self];

     [survMgrReddMap at: species insert: survMgr];

     ANIMAL = [survMgr getANIMALSYMBOL];
     HABITAT = [survMgr getHABITATSYMBOL];

    //
    // Dewatering
    //
    [survMgr addPROBWithSymbol: [model getReddMortalitySymbolWithName: "ReddDewater"]
                      withType: "SingleFunctionProb"
                withAgentKnows: YES
               withIsStarvProb: NO];

    [survMgr addBoolSwitchFuncToProbWithSymbol: [model getReddMortalitySymbolWithName: "ReddDewater"]
                           withInputObjectType: HABITAT
                             withInputSelector: M(isDepthGreaterThan0)
                                  withYesValue: 1.0
		                   withNoValue: fishParams->mortReddDewaterSurv];


     //
     // Scouring
     // 
     [survMgr addPROBWithSymbol: [model getReddMortalitySymbolWithName: "ReddScour"] 
                       withType: "CustomProb"
                 withAgentKnows: YES
                withIsStarvProb: NO];
  
   
     [survMgr addCustomFuncToProbWithSymbol: [model getReddMortalitySymbolWithName: "ReddScour"] 
                              withClassName: "ReddScourFunc"
                        withInputObjectType: ANIMAL
                          withInputSelector: M(getCell)];


     //
     // Low Temperature
     //
     [survMgr addPROBWithSymbol: [model getReddMortalitySymbolWithName: "LowTemperature"]
                       withType: "SingleFunctionProb"
                 withAgentKnows: YES
                withIsStarvProb: NO];


     [survMgr addLogisticFuncToProbWithSymbol: [model getReddMortalitySymbolWithName: "LowTemperature"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getTemperature)
                                  withXValue1: fishParams->mortReddLoTT1
                                  withYValue1: 0.1
                                  withXValue2: fishParams->mortReddLoTT9
                                  withYValue2: 0.9];




     //
     // High Temperature
     //
     [survMgr addPROBWithSymbol: [model getReddMortalitySymbolWithName: "HighTemperature"]
                       withType: "SingleFunctionProb"
                 withAgentKnows: YES
                withIsStarvProb: NO];


     [survMgr addLogisticFuncToProbWithSymbol: [model getReddMortalitySymbolWithName: "HighTemperature"] 
                          withInputObjectType: HABITAT
                            withInputSelector: M(getTemperature)
                                  withXValue1: fishParams->mortReddHiTT9
                                  withYValue1: 0.9
                                  withXValue2: fishParams->mortReddHiTT1
                                  withYValue2: 0.1];

     //
     // SuperImposition
     //
     
      [survMgr addPROBWithSymbol: [model getReddMortalitySymbolWithName: "ReddSuperImp"] 
                       withType: "CustomProb"
                 withAgentKnows: YES
                withIsStarvProb: NO];
  
   
      [survMgr addCustomFuncToProbWithSymbol: [model getReddMortalitySymbolWithName: "ReddSuperImp"] 
                               withClassName: "ReddSuperImpFunc"
                         withInputObjectType: ANIMAL
                           withInputSelector: M(getCell)];


     [survMgr setLogisticFuncLimiterTo: 20.0];
     //[survMgr setTestOutputOnWithFileName: "SurvMGRTest.out"];
     survMgr = [survMgr createEnd];
  }
 
  //fprintf(stdout, "Cell >>>> initializeSurvProb >>>> END\n");
  //fflush(0);

  return self;
}



//////////////////////////////////////////////
//
////  Survival Manager code for the cell
/////
///////
/////////
/////////////////////////////////////////////

/////////////////////////////////////////////////////
//
// updateHabitatSurvivalProb
//
/////////////////////////////////////////////////////
- updateHabitatSurvivalProb {

  [survMgrMap forEach: M(updateForHabitat)];
  [survMgrReddMap forEach: M(updateForHabitat)];

  return self;
}


//////////////////////////////////
//
// updateHabSurvProbForAqPred
//
/////////////////////////////////
- updateHabSurvProbForAqPred
{
  [survMgrMap forEach: M(updateForHabitat)];

  return self;
}


- updateFishSurvivalProbFor: aFish
{
   [[survMgrMap at: [aFish getSpecies]] 
          updateForAnimal: aFish]; 

   return self;
}


- updateReddSurvivalProbFor: aRedd
{
   [[survMgrReddMap at: [aRedd getSpecies]] 
               updateForAnimal: aRedd]; 

   return self;
}



//////////////////////////////////////////
//
//(id <List>) getListOfSurvProbsFor: aFish
//
//////////////////////////////////////////
- (id <List>) getListOfSurvProbsFor: aFish
{
   //fprintf(stdout, "CELL >>>> getListOfSurvProbsFor >>>> BEGIN\n");
   //fprintf(stdout, "CELL >>>> offset = %d\n", [aFish getSpeciesNdx]);
   //fflush(0);

   return [[survMgrMap at: [aFish getSpecies]] getListOfSurvProbsFor: aFish]; 
}

//////////////////////////////////////////
//
//(id <List>) getReddListOfSurvProbsFor: aRedd
//
//////////////////////////////////////////
- (id <List>) getReddListOfSurvProbsFor: aRedd
{
   //fprintf(stdout, "CELL >>>> getReddListOfSurvProbsFor >>>> BEGIN\n");
   //fprintf(stdout, "CELL >>>> offset = %d\n", [aRedd getSpeciesNdx]);
   //fflush(0);

   return [[survMgrReddMap at: [aRedd getSpecies]] getListOfSurvProbsFor: aRedd]; 
}



- (double) getTotalKnownNonStarvSurvivalProbFor: aFish
{
  return  [[survMgrMap at: [aFish getSpecies]] getTotalKnownNonStarvSurvivalProbFor: aFish];
}



- (double) getStarvSurvivalFor: aFish
{
   return [[survMgrMap at: [aFish getSpecies]] 
           getStarvSurvivalFor: aFish]; 
}


#ifdef FOODAVAILREPORT

- foodAvailAndConInCell: aFish {

  FILE * foodReportPtr=NULL;
  const char * foodReportFile = "FoodAvailability.rpt";


  if([space getFoodReportFirstTime] == YES) {

     if( (foodReportPtr = fopen(foodReportFile,"w")) == NULL) {
       [InternalError raiseEvent:"ERROR: Cannot open %s for writing",foodReportFile];
      }


        fprintf(foodReportPtr,"\n%-15s%-10s%-16s%-16s%-16s%-16s%-16s%-16s%-16s\n","Date",
                                                         "Transect",
                                                         "CellNo",
                                                         "SearchFoodProd",
                                                         "DriftFoodProd",
                                                         "SearchAvail",
                                                         "Driftavail",
                                                         "SearchConsumed",
                                                         "DriftConsumed");
  fflush(foodReportPtr);

   }

  if( [space getFoodReportFirstTime] == NO) {

     if( (foodReportPtr = fopen(foodReportFile,"a")) == NULL) {
       [InternalError raiseEvent:"ERROR: Cannot open %s for writing",foodReportFile];
      }

  fprintf(foodReportPtr,"%-15s%-10d%-16d%-16E%-16E%-16E%-16E%-16E%-16E\n",[timeManager getDateWithTimeT: [space getModelTime]],
                                                         transectNumber,
                                                         cellNumber,
                                                         searchHourlyCellTotal,
                                                         driftHourlyCellTotal,
                                                         hourlyAvailSearchFood,
                                                         hourlyAvailDriftFood,
                                                [aFish getHourlySearchConRate],
                                                [aFish getHourlyDriftConRate]);

  fflush(foodReportPtr);
}
  if(foodReportPtr != NULL) {
   fclose(foodReportPtr);

[space setFoodReportFirstTime: NO];
return self;
}


  return self;
}

#endif



- depthVelReport: (FILE *) depthVelPtr {

 if([space getDepthVelRptFirstTime] == YES) {
   fprintf(depthVelPtr,"%-15s%-15s%-16s%-7s%-16s%-16s%-16s\n", "Date",
                                                        "Flow",
                                                        "TransectNumber",
                                                        "CellNo",
                                                        "CellArea",
                                                        "Depth",
                                                        "Velocity");

  }

if(depth != 0) {
 fprintf(depthVelPtr,"%-15s%-15f%-16d%-7d%-16f%-16f%-16f\n",[timeManager getDateWithTimeT: [space getModelTime]],
                                                     [space getRiverFlow],
                                                     transectNumber,
                                                     cellNumber,
                                                     area,
                                                     depth,
                                                     velocity);
}
 
 [space setDepthVelRptFirstTime: NO];

 return self;
}



///////////////////////////////////////////////////////////
//
// isThereABarrierTo
//
/////////////////////////////////////////////////////////
- (int) isThereABarrierTo: aCell 
{
   //
   //returns -1, 0, 1 depending on whether the barrier is downstream 
   //
   return [space isThereABarrierTo: aCell from: self];
}    



/////////////////////////////////////////
//
// drop
//
////////////////////////////////////////
- (void) drop
{
   [cellZone drop];
}

@end
