/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/


#import <random.h>
#import <space.h>
#import <space/Discrete2d.h>
#import "ZoneAllocMapper.h"
#import "Cell.h"
#import "globals.h"
#import "BreakoutReporter.h"
#import "TimeManagerProtocol.h"
#import "TimeSeriesInputManagerP.h"
#import "HydraulicFileReader.h"
#import "StationObject.h"
#import "Barrier.h"
#import "math.h"
#import "InterpolationTableP.h"
#import "SolarManagerP.h"
#import "TroutModelSwarmP.h"

@interface HabitatSpace : Discrete2d 
{
  id <TroutModelSwarm> model;
  id <Zone> habitatZone;

  char* reachName;
  char* instanceName;
  id <Symbol> reachSymbol;

  id <SolarManager> solarManager;

  int numCells;
  id <List> listOfCells;
  id <ListIndex> listOfCellsNdx;
  unsigned pixelsX;
  unsigned pixelsY;

  //
  // Added for adjacent reaches
  //
  int habDownstreamJunctionNumber;
  int habUpstreamJunctionNumber;

  id <List> habDownstreamLinksToDS;
  id <List> habDownstreamLinksToUS;

  id <List> habUpstreamLinksToDS;
  id <List> habUpstreamLinksToUS;

  id <List> tempCellList;
  
  //
  // End adjacent reaches
  //



  //
  //THE FOLLOWING VARIABLES ARE INITIALIZED BY Habitat.Setup
  //
  double habSearchProd;
  double habDriftConc;
  double habDriftRegenDist;
  double habPreyEnergyDensity;
  double habMaxSpawnFlow;
  double habShearParamA;
  double habShearParamB;
  double habShelterSpeedFrac;

//END VARIABLES INITIALIZED BY Habitat.Setup


@public
  int rasterResolution;
  int rasterResolutionX;
  int rasterResolutionY;
  char * rasterColorVariable;



@protected

  int numberOfSpecies;



  //
  // PRED DENSITY
  // needed by AQUATIC SURV PROB
  //
  double streamLength;
  double reachLength;

  double habWettedArea;

  int numPiscivorousFish;
  double piscivorousFishDensity;


  //
  // These are set from the HabitatManager 
  //
  char* cellDataFile;
  char* flowFile;
  char* temperatureFile;
  char* turbidityFile;
  char* hydraulicsFile;

  id <TimeSeriesInputManager> flowInputManager;
  id <TimeSeriesInputManager> temperatureInputManager;
  id <TimeSeriesInputManager> turbidityInputManager;

  double temperature;
  double turbidity;
  
  int spaceDimX, spaceDimY;


  //
  // BEGIN: New Hydraulic Input File variables
  //

  HydraulicFileReader *hydraulicFileReader;

  id <List> stationList;
  id <Array> flowArray;
  id <InterpolationTable> flowInterpolator;


  //
  // END: New Hydraulic Input File variables
  //


  // 
  // BARRIERS
  //
  id <List> barrierList;
  int barrierCount;
  double maxCellY;
  double minCellY;

  double dayLength;

  id <LogNormalDist> velDist;

  double yesterdaysRiverFlow;
  double riverFlow;
  double tomorrowsRiverFlow;

  double flowChange;

 
  //
  // TIME
  // 
 
  id <TimeManager> timeManager;
  time_t modelTime_t;
  char* modelDate;
  time_t modelStartTime;
  time_t modelEndTime;

  time_t dataStartTime;
  time_t dataEndTime;

  char* Date;

  int scenario;
  int replicate;

  id <Map> fishParamsMap;

  //
  // REPORT VARIABLES
  //

 BOOL appendFiles;

 BOOL habitatReportFirstWrite;
 BOOL depthReportFirstWrite;
 BOOL velocityReportFirstWrite;
 BOOL depthVelRptFirstTime;     //also used in the cell depth reporting


 char* cellDepthReportFile;
 char* cellVelocityReportFile;
 char* habitatReportFile;
 char* cellAreaDepthVelReportFile;




  //  
  // CELL REPORT VARIABLES
  // 
  BOOL foodReportFirstTime;

  BreakoutReporter* cellFishInfoReporter;
  char cellFishInfoReportFName[50];
  id <List> cellFishList;
  id <List> ageSymbolList;
  id <List> speciesSymbolList;
  id <List> reachSymbolList;

  //
  // The following is used for a work around in the 
  // cellFishInfoReporter (BreakoutReport)
  //
  int    habCellTransectNumber;
  int    habCellNumber;
  double habCellArea;
  double habCellDepth;
  double habCellVelocity;
  double habCellDistToHide;
  double habCellFracShelter;


  //
  // Habitat Report
  //
  FILE* habitatRptFilePtr;
  //
  // HISTOGRAM VARIABLES
  //

  FILE* areaDepthFileStream;
  FILE* areaVelocityFileStream;

  int depthBinWidth;
  int velocityBinWidth;

  double depthHistoMaxDepth;
  double velocityHistoMaxVelocity;

  BOOL firstDepthTime;
  BOOL firstVelocityTime;

  char* areaDepthHistoFmtStr;
  char* areaVelocityHistoFmtStr;

}

+ createBegin: aZone;
- createEnd;
- checkHabitatParams;

- buildObjects;
- finishBuildObjects;

- setModel: (id <TroutModelSwarm>) aModel;
- getModel;

- setReachName: (char *) aReachName;
- setInstanceName: (char *) anInstanceName;
- (char *) getReachName;
- (char *) getInstanceName;

- setReachSymbol: (id <Symbol>) aSymbol;
- (id <Symbol>) getReachSymbol;


//
// Adjacent Cells BEGIN
//
- setHabDStreamJNumber: (int) aJunctionNumber;
- setHabUStreamJNumber: (int) aJunctionNumber;
- (int) getHabDStreamJNumber;
- (int) getHabUStreamJNumber;

- setHabDownstreamLinksToDS: aDSLinkToDS;
- setHabDownstreamLinksToUS: aDSLinkToUS;
- setHabUpstreamLinksToDS: anUSLinkToDS;
- setHabUpstreamLinksToUS: anUSLinkToUS;

- (id <List>) getHabDownstreamLinksToDS;
- (id <List>) getHabDownstreamLinksToUS;
- (id <List>) getHabUpstreamLinksToDS;
- (id <List>) getHabUpstreamLinksToUS;

- (id <List>) getDownstreamCellsWithinRange: (double) aRange;
- (id <List>) getUpstreamCellsWithinRange: (double) aRange;

- (double) getReachLength;

     
//
// Adjacent Cells END
//


- setTimeManager: (id <TimeManager>) aTimeManager;
- setFishParamsMap: (id <Map>) aMap;
- setSolarManager: (id <SolarManager>) aSolarManager;
- setNumberOfSpecies: (int) aNumberOfSpecies;

- (id <List>) getCells;

- getNeighborsWithin: (double) aRange 
                              of: refCell 
                        withList: aCellList; // used by fish

- getAdjacentCells: refCell;
- readGeometry;
- calcSpaceVariables;



- setCellDataFile: (char *) aFile;
- setFlowFile: (char *) aFile;
- setTemperatureFile: (char *) aFile;
- setTurbidityFile: (char *) aFile;
- createTimeSeriesInputManagers;


- setSpaceDimensions;
- (int) getSpaceDimX;
- (int) getSpaceDimY;

//- (int) getDate;
//- (int) getYear;
- (time_t) getModelTime;



- probeCellAtX: (int) probedX Y: (int) probedY;
- probeFishAtX: (int) probedX Y: (int) probedY;
- _getCellContainingX: (int) probedX Y: (int) probedY;


- buildCells;

- setModelStartTime: (time_t) startTime 
         andEndTime: (time_t) endTime;

- setDataStartTime: (time_t) aDataStartTime
        andEndTime: (time_t) aDataEndTime;


// 
// UPDATE
// 
-   updateHabitatWithTime: (time_t) aModelTime_t
    andWithModelStartFlag: (BOOL) aStartFlag;
- updateCellHydraulicsWith: (double) aFlow;
- calcWettedArea;
- updateFlowChange;



- (double) getTemperature;

// retrieve the total pixels covered by the Cells in X and Y
- (unsigned)getPixelsX;
- (unsigned)getPixelsY;

- (double) readTomorrowsFlow: (time_t) aModelTime_t;

- (double) getFlowChange;
- (double) getYesterdaysRiverFlow;
- (double) getRiverFlow;
- (double) getTomorrowsRiverFlow;

- (double) getHabMaxSpawnFlow;

- (double) getDayLength;

- (double) getHabSearchProd;
- (double) getHabDriftConc;
- (double) getHabDriftRegenDist;
- (double) getHabPreyEnergyDensity;

- buildCellFishInfoReporter;
- outputCellFishInfoReport;

- printCellDepthReport;
- printCellVelocityReport;

- printHabitatReport;
- printCellAreaDepthVelocityRpt;

- (BOOL) getFoodReportFirstTime;
- setFoodReportFirstTime: (BOOL) aBool;
- (BOOL) getDepthVelRptFirstTime;
- setDepthVelRptFirstTime: (BOOL) aBool;

//
// HYDRAULIC INPUT FILE METHODS
//
- readHydraulicInputFile: (char *) inputFile;
- createCellInterpolationTables;
- setCellHydraulics;

//
// BARRIERS
//
- buildBarriersFromList: (id <List>) aBarrierXList;


- setBarrierRasterResolutionX: (int) aResolutionX 
               andResolutionY: (int) aResolutionY;


- (id <List>) getBarriers;
- (int) isThereABarrierTo: aDestCell from: myCell;
- (BOOL) hasBarrierBetweenDownstreamEndAnd: aCell;
- (BOOL) hasBarrierBetweenUpstreamEndAnd: aCell;




//
//  PRED DENSITY
//

- incrementNumPiscivorousFish;
- updateHabSurvProbForAqPred;
- (double) calcPiscivorousFishDensity;
- (double) getPiscivorousFishDensity;

//
// NEW ADJACENT CELL
//
- _getCellContainingFloatX: (double) probedX floatY: (double) probedY;

//
// HISTOGRAM
//
- setAreaDepthBinWidth: (int) aWidth;
- setDepthHistoMaxDepth: (double) aDepth;
- setAreaVelocityBinWidth: (int) aWidth;
- setVelocityHistoMaxVelocity: (double) aVelocity;

- setAreaDepthHistoFmtStr: (char *) aFmtStr;
- setAreaVelocityHistoFmtStr: (char *) aFmtStr;

- openAreaDepthFile: (char *) aFileName;
- openAreaVelocityFile: (char *) aFileName;

- printAreaDepthHisto;
- printAreaVelocityHisto;


//
// CLEANUP
//
- (void) drop;




@end
